'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _immutable = require('immutable');

var _relationshipTypes = require('./relationshipTypes');

exports.default = {
  link: function link(mapOfRelationshipTypes) {
    return function (state, _ref) {
      var payload = _ref.payload,
          error = _ref.error;

      if (error) {
        return state;
      }
      var _payload$relationship = payload.relationship,
          relationshipName = _payload$relationship.relationshipName,
          id = _payload$relationship.id,
          relationshipValue = _payload$relationship.relationshipValue;

      var relationshipType = mapOfRelationshipTypes[relationshipName];
      if (relationshipType === _relationshipTypes.MANY) {
        return state.updateIn([relationshipName, '' + id], function (ids) {
          if (ids) {
            return ids.add(relationshipValue);
          }
          return new _immutable.OrderedSet([relationshipValue]);
        });
      }
      return state.setIn([relationshipName, '' + id], relationshipValue);
    };
  },
  createRelationship: function createRelationship(mapOfRelationshipTypes) {
    return function (state, _ref2) {
      var payload = _ref2.payload,
          error = _ref2.error;

      if (error) {
        return state;
      }
      var _payload$relationship2 = payload.relationship,
          relationshipName = _payload$relationship2.relationshipName,
          id = _payload$relationship2.id,
          relationshipValue = _payload$relationship2.relationshipValue;

      var relationshipType = mapOfRelationshipTypes[relationshipName];
      if (relationshipType === _relationshipTypes.MANY) {
        return state.setIn([relationshipName, '' + id], new _immutable.OrderedSet([relationshipValue]));
      }
      return state.setIn([relationshipName, '' + id], relationshipValue);
    };
  },
  unlink: function unlink(mapOfRelationshipTypes) {
    return function (state, _ref3) {
      var payload = _ref3.payload,
          error = _ref3.error;

      if (error) {
        return state;
      }
      var _payload$relationship3 = payload.relationship,
          relationshipName = _payload$relationship3.relationshipName,
          id = _payload$relationship3.id,
          relationshipValue = _payload$relationship3.relationshipValue;

      var relationshipType = mapOfRelationshipTypes[relationshipName];
      if (relationshipType === _relationshipTypes.MANY) {
        return state.updateIn([relationshipName, '' + id], function (ids) {
          if (ids) {
            return ids.remove(relationshipValue);
          }
          return new _immutable.OrderedSet();
        });
      }
      return state.setIn([relationshipName, '' + id], 0);
    };
  },
  indexRelationship: function indexRelationship(mapOfRelationshipTypes) {
    return function (state, _ref4) {
      var payload = _ref4.payload,
          error = _ref4.error;

      if (error) {
        return state;
      }
      var _payload$relationship4 = payload.relationships,
          relationshipName = _payload$relationship4.relationshipName,
          idValuePairs = _payload$relationship4.idValuePairs;

      var relationshipType = mapOfRelationshipTypes[relationshipName];
      return state.updateIn([relationshipName], function (relationships) {
        idValuePairs.reduce(function (finalResult, _ref5) {
          var id = _ref5.id,
              value = _ref5.value;

          var finalValue = relationshipType === _relationshipTypes.MANY ? new _immutable.OrderedSet(value) : value;
          finalResult.set('id', finalValue);
          return finalResult;
        }, relationships);
      });
    };
  },
  remove: function remove(mapOfRelationshipTypes, mapOfRelationships, entityName) {
    return function (state, _ref6) {
      var payload = _ref6.payload,
          error = _ref6.error;

      if (error) {
        return state;
      }
      var id = payload.id;

      var relationshipNames = mapOfRelationships[entityName];
      return relationshipNames.reduce(function (finalResult, relationshipName) {
        var relationshipType = mapOfRelationshipTypes[relationshipName];
        if (relationshipType === _relationshipTypes.MANY) {
          return finalResult.updateIn([relationshipName], function (mapOfIds) {
            return mapOfIds.map(function (ids) {
              return ids && ids.delete(id);
            });
          });
        }
        return finalResult.updateIn([relationshipName], function (mapOfIds) {
          return mapOfIds.map(function (idValue) {
            return idValue === id ? 0 : idValue;
          });
        });
      }, state);
    };
  }
};